/* include/SAX/ArabicaConfig.hpp.  Generated from ArabicaConfig.hpp.in by configure.  */
/* include/SAX/ArabicaConfig.hpp.in.  Generated from configure.ac by autoheader.  */

/* defined if we don't need extra codecvt template specialisations */
/* #undef ARABICA_NO_CODECVT_SPECIALISATIONS */

/* no mbstate_t */
/* #undef ARABICA_NO_MBSTATE_T */

/* no std::mbstate_t */
/* #undef ARABICA_NO_STD_MBSTATE_T */

/* disables wchar_t */
/* #undef ARABICA_NO_WCHAR_T */

/* defined for Windows builds using Winsock rather than BSD-style sockets */
/* #undef ARABICA_USE_WINSOCK */

/* define if the Boost library is available */
#define HAVE_BOOST /**/

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* define to use the Elephant memory leak detector */
/* #undef HAVE_ELEPHANT */

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to the sub-directory in which libtool stores uninstalled libraries.
   */
#define LT_OBJDIR ".libs/"

/* Name of package */
#define PACKAGE "arabica"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "jez@jezuk.co.uk"

/* Define to the full name of this package. */
#define PACKAGE_NAME "Arabica"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "Arabica 2010-November"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "arabica"

/* Define to the home page for this package. */
#define PACKAGE_URL ""

/* Define to the version of this package. */
#define PACKAGE_VERSION "2010-November"

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* define to build against Expat */
#define USE_EXPAT /**/

/* define to build using Garden */
/* #undef USE_GARDEN */

/* define to build against LibXML2 */
/* #undef USE_LIBXML2 */

/* define to build against Xerces */
/* #undef USE_XERCES */

/* Version number of package */
#define VERSION "2010-November"
