#ifndef ARABICA_TAGGLE_TAGGLE_HPP
#define ARABICA_TAGGLE_TAGGLE_HPP

#include "impl/ScanHandler.hpp"
#include "impl/ElementType.hpp"
#include "impl/Element.hpp"
#include "impl/Schema.hpp"
#include "impl/html/HTMLModels.hpp"
#include "impl/html/HTMLScanner.hpp"
#include "impl/html/HTMLSchema.hpp"
#include "impl/Parser.hpp"

#endif 
